@@ -83,7 +83,6 @@ pattern RPL_STATSPLINE              = ReplyCode 220
 pattern RPL_UMODEIS                 = ReplyCode 221
 pattern RPL_SQLINE_NICK             = ReplyCode 222
 pattern RPL_STATSDLINE              = ReplyCode 225
-pattern RPL_STATSZLINE              = ReplyCode 225
 pattern RPL_STATSCOUNT              = ReplyCode 226
 pattern RPL_SERVICEINFO             = ReplyCode 231
 pattern RPL_ENDOFSERVICES           = ReplyCode 232
@@ -149,6 +148,7 @@ pattern RPL_WHOISADMIN              = ReplyCode 308
 pattern RPL_NICKTRACE               = ReplyCode 309
 pattern RPL_WHOISSADMIN             = ReplyCode 309
 pattern RPL_WHOISHELPER             = ReplyCode 309
+pattern RPL_WHOISHELPOP             = ReplyCode 310
 pattern RPL_WHOISUSER               = ReplyCode 311
 pattern RPL_WHOISSERVER             = ReplyCode 312
 pattern RPL_WHOISOPERATOR           = ReplyCode 313
@@ -168,7 +168,6 @@ pattern RPL_NOCHANPASS              = ReplyCode 326
 pattern RPL_CHPASSUNKNOWN           = ReplyCode 327
 pattern RPL_CHANNEL_URL             = ReplyCode 328
 pattern RPL_CREATIONTIME            = ReplyCode 329
-pattern RPL_WHOWAS_TIME             = ReplyCode 330
 pattern RPL_WHOISACCOUNT            = ReplyCode 330
 pattern RPL_NOTOPIC                 = ReplyCode 331
 pattern RPL_TOPIC                   = ReplyCode 332
@@ -229,11 +228,12 @@ pattern ERR_TOOMANYCHANNELS         = ReplyCode 405
 pattern ERR_WASNOSUCHNICK           = ReplyCode 406
 pattern ERR_TOOMANYTARGETS          = ReplyCode 407
 pattern ERR_NOORIGIN                = ReplyCode 409
+pattern ERR_INVALIDCAPCMD           = ReplyCode 410
 pattern ERR_NORECIPIENT             = ReplyCode 411
 pattern ERR_NOTEXTTOSEND            = ReplyCode 412
 pattern ERR_NOTOPLEVEL              = ReplyCode 413
 pattern ERR_WILDTOPLEVEL            = ReplyCode 414
-pattern ERR_BADMASK                 = ReplyCode 415
+pattern ERR_MSGNEEDREGGEDNICK       = ReplyCode 415
 pattern ERR_TOOMANYMATCHES          = ReplyCode 416
 pattern ERR_LENGTHTRUNCATED         = ReplyCode 419
 pattern ERR_UNKNOWNCOMMAND          = ReplyCode 421
@@ -368,7 +368,7 @@ pattern RPL_CHANREDIR               = ReplyCode 663
 pattern RPL_SERVMODEIS              = ReplyCode 664
 pattern RPL_OTHERUMODEIS            = ReplyCode 665
 pattern RPL_ENDOF_GENERIC           = ReplyCode 666
-pattern RPL_WHOWASDETAILS           = ReplyCode 670
+pattern RPL_STARTTLS                = ReplyCode 670
 pattern RPL_WHOISSECURE             = ReplyCode 671
 pattern RPL_UNKNOWNMODES            = ReplyCode 672
 pattern RPL_CANNOTSETMODES          = ReplyCode 673
@@ -379,6 +379,7 @@ pattern RPL_YOURLANGUAGEIS          = ReplyCode 687
 pattern RPL_LANGUAGE                = ReplyCode 688
 pattern RPL_WHOISSTAFF              = ReplyCode 689
 pattern RPL_WHOISLANGUAGE           = ReplyCode 690
+pattern ERR_STARTTLS                = ReplyCode 691
 pattern RPL_MODLIST                 = ReplyCode 702
 pattern RPL_ENDOFMODLIST            = ReplyCode 703
 pattern RPL_HELPSTART               = ReplyCode 704
@@ -423,10 +424,10 @@ pattern RPL_LOGGEDIN                = ReplyCode 900
 pattern RPL_LOGGEDOUT               = ReplyCode 901
 pattern RPL_NICKLOCKED              = ReplyCode 902
 pattern RPL_SASLSUCCESS             = ReplyCode 903
-pattern RPL_SASLFAIL                = ReplyCode 904
-pattern RPL_SASLTOOLONG             = ReplyCode 905
-pattern RPL_SASLABORTED             = ReplyCode 906
-pattern RPL_SASLALREADY             = ReplyCode 907
+pattern ERR_SASLFAIL                = ReplyCode 904
+pattern ERR_SASLTOOLONG             = ReplyCode 905
+pattern ERR_SASLABORTED             = ReplyCode 906
+pattern ERR_SASLALREADY             = ReplyCode 907
 pattern RPL_SASLMECHS               = ReplyCode 908
 pattern ERR_CANNOTDOCOMMAND         = ReplyCode 972
 pattern ERR_CANNOTCHANGEUMODE       = ReplyCode 973
@@ -463,7 +464,7 @@ replyCodeInfo (ReplyCode w) =
 defaultReplyCodeInfo :: Int -> ReplyCodeInfo
 defaultReplyCodeInfo = ReplyCodeInfo UnknownReply . Text.pack . show
 
--- | Information about reply codes as derived from Freenode's ircd-seven.
+-- | Information about reply codes as derived from ircd-seven.
 replyCodeInfoTable :: Vector ReplyCodeInfo
 replyCodeInfoTable
   = Vector.accumulate
@@ -512,7 +513,6 @@ replyCodeInfoTable
   , (RPL_UMODEIS               , ReplyCodeInfo CommandReply "umode-is")
   , (RPL_SQLINE_NICK           , ReplyCodeInfo CommandReply "sqline-nick")
   , (RPL_STATSDLINE            , ReplyCodeInfo CommandReply "stats-dline")
-  , (RPL_STATSZLINE            , ReplyCodeInfo CommandReply "stats-zline")
   , (RPL_STATSCOUNT            , ReplyCodeInfo CommandReply "stats-count")
   , (RPL_SERVICEINFO           , ReplyCodeInfo CommandReply "service-info")
   , (RPL_ENDOFSERVICES         , ReplyCodeInfo CommandReply "end-of-services")
@@ -542,7 +542,7 @@ replyCodeInfoTable
   , (RPL_ADMINEMAIL            , ReplyCodeInfo CommandReply "admin-email")
   , (RPL_TRACELOG              , ReplyCodeInfo CommandReply "trace-log")
   , (RPL_ENDOFTRACE            , ReplyCodeInfo CommandReply "end-of-trace")
-  , (RPL_LOAD2HI               , ReplyCodeInfo CommandReply "load-too-hi")
+  , (RPL_LOAD2HI               , ReplyCodeInfo CommandReply "load-too-high")
   , (RPL_LOCALUSERS            , ReplyCodeInfo CommandReply "local-users")
   , (RPL_GLOBALUSERS           , ReplyCodeInfo CommandReply "global-users")
   , (RPL_START_NETSTAT         , ReplyCodeInfo CommandReply "start-netstat")
@@ -570,7 +570,7 @@ replyCodeInfoTable
   , (RPL_ISON                  , ReplyCodeInfo CommandReply "ison")
   , (RPL_TEXT                  , ReplyCodeInfo CommandReply "text")
   , (RPL_UNAWAY                , ReplyCodeInfo CommandReply "unaway")
-  , (RPL_NOWAWAY               , ReplyCodeInfo CommandReply "nowaway")
+  , (RPL_NOWAWAY               , ReplyCodeInfo CommandReply "now-away")
   , (RPL_WHOISREGNICK          , ReplyCodeInfo CommandReply "whois-regnick")
   , (RPL_SUSERHOST             , ReplyCodeInfo CommandReply "suserhost")
   , (RPL_NOTIFYACTION          , ReplyCodeInfo CommandReply "notify-action")
@@ -578,6 +578,7 @@ replyCodeInfoTable
   , (RPL_NICKTRACE             , ReplyCodeInfo CommandReply "nick-trace")
   , (RPL_WHOISSADMIN           , ReplyCodeInfo CommandReply "whois-sadmin")
   , (RPL_WHOISHELPER           , ReplyCodeInfo CommandReply "whois-helper")
+  , (RPL_WHOISHELPOP           , ReplyCodeInfo CommandReply "whois-helpop")
   , (RPL_WHOISUSER             , ReplyCodeInfo CommandReply "whois-user")
   , (RPL_WHOISSERVER           , ReplyCodeInfo CommandReply "whois-server")
   , (RPL_WHOISOPERATOR         , ReplyCodeInfo CommandReply "whois-operator")
@@ -597,7 +598,6 @@ replyCodeInfoTable
   , (RPL_CHPASSUNKNOWN         , ReplyCodeInfo CommandReply "chpass-unknown")
   , (RPL_CHANNEL_URL           , ReplyCodeInfo CommandReply "channel-url")
   , (RPL_CREATIONTIME          , ReplyCodeInfo CommandReply "creation-time")
-  , (RPL_WHOWAS_TIME           , ReplyCodeInfo CommandReply "whowas-time")
   , (RPL_WHOISACCOUNT          , ReplyCodeInfo CommandReply "whois-account")
   , (RPL_NOTOPIC               , ReplyCodeInfo CommandReply "notopic")
   , (RPL_TOPIC                 , ReplyCodeInfo CommandReply "topic")
@@ -658,11 +658,12 @@ replyCodeInfoTable
   , (ERR_WASNOSUCHNICK         , ReplyCodeInfo ErrorReply "was-no-such-nick")
   , (ERR_TOOMANYTARGETS        , ReplyCodeInfo ErrorReply "too-many-targets")
   , (ERR_NOORIGIN              , ReplyCodeInfo ErrorReply "no-origin")
+  , (ERR_INVALIDCAPCMD         , ReplyCodeInfo ErrorReply "invalid-cap-cmd")
   , (ERR_NORECIPIENT           , ReplyCodeInfo ErrorReply "no-recipient")
   , (ERR_NOTEXTTOSEND          , ReplyCodeInfo ErrorReply "no-text-to-send")
   , (ERR_NOTOPLEVEL            , ReplyCodeInfo ErrorReply "no-top-level")
   , (ERR_WILDTOPLEVEL          , ReplyCodeInfo ErrorReply "wild-top-level")
-  , (ERR_BADMASK               , ReplyCodeInfo ErrorReply "bad-mask")
+  , (ERR_MSGNEEDREGGEDNICK     , ReplyCodeInfo ErrorReply "msg-need-regged-nick")
   , (ERR_TOOMANYMATCHES        , ReplyCodeInfo ErrorReply "too-many-matches")
   , (ERR_LENGTHTRUNCATED       , ReplyCodeInfo ErrorReply "length-truncated")
   , (ERR_UNKNOWNCOMMAND        , ReplyCodeInfo ErrorReply "unknown-command")
@@ -797,7 +798,7 @@ replyCodeInfoTable
   , (RPL_SERVMODEIS            , ReplyCodeInfo CommandReply "serv-mode-is")
   , (RPL_OTHERUMODEIS          , ReplyCodeInfo CommandReply "other-umode-is")
   , (RPL_ENDOF_GENERIC         , ReplyCodeInfo CommandReply "end-of-generic")
-  , (RPL_WHOWASDETAILS         , ReplyCodeInfo CommandReply "whowas-details")
+  , (RPL_STARTTLS              , ReplyCodeInfo CommandReply "start-tls")
   , (RPL_WHOISSECURE           , ReplyCodeInfo CommandReply "whois-secure")
   , (RPL_UNKNOWNMODES          , ReplyCodeInfo CommandReply "unknown-modes")
   , (RPL_CANNOTSETMODES        , ReplyCodeInfo CommandReply "cannot-set-modes")
@@ -808,6 +809,7 @@ replyCodeInfoTable
   , (RPL_LANGUAGE              , ReplyCodeInfo CommandReply "language")
   , (RPL_WHOISSTAFF            , ReplyCodeInfo CommandReply "whois-staff")
   , (RPL_WHOISLANGUAGE         , ReplyCodeInfo CommandReply "who-is-language")
+  , (ERR_STARTTLS              , ReplyCodeInfo CommandReply "err-start-tls")
   , (RPL_MODLIST               , ReplyCodeInfo CommandReply "mod-list")
   , (RPL_ENDOFMODLIST          , ReplyCodeInfo CommandReply "end-of-modlist")
   , (RPL_HELPSTART             , ReplyCodeInfo CommandReply "help-start")
@@ -852,10 +854,10 @@ replyCodeInfoTable
   , (RPL_LOGGEDOUT             , ReplyCodeInfo CommandReply "logged-out")
   , (RPL_NICKLOCKED            , ReplyCodeInfo CommandReply "nick-locked")
   , (RPL_SASLSUCCESS           , ReplyCodeInfo CommandReply "sasl-success")
-  , (RPL_SASLFAIL              , ReplyCodeInfo CommandReply "sasl-fail")
-  , (RPL_SASLTOOLONG           , ReplyCodeInfo CommandReply "sasl-toolong")
-  , (RPL_SASLABORTED           , ReplyCodeInfo CommandReply "sasl-aborted")
-  , (RPL_SASLALREADY           , ReplyCodeInfo CommandReply "sasl-already")
+  , (ERR_SASLFAIL              , ReplyCodeInfo ErrorReply "sasl-fail")
+  , (ERR_SASLTOOLONG           , ReplyCodeInfo ErrorReply "sasl-toolong")
+  , (ERR_SASLABORTED           , ReplyCodeInfo ErrorReply "sasl-aborted")
+  , (ERR_SASLALREADY           , ReplyCodeInfo ErrorReply "sasl-already")
   , (RPL_SASLMECHS             , ReplyCodeInfo CommandReply "sasl-mechs")
   , (ERR_CANNOTDOCOMMAND       , ReplyCodeInfo ErrorReply "cannot-do-command")
   , (ERR_CANNOTCHANGEUMODE     , ReplyCodeInfo ErrorReply "cannot-change-umode")
