@@ -25,11 +25,14 @@ module Irc.Message
   , ircMsgText
   , msgTarget
   , msgActor
+  , msgSource
 
   -- * Helper functions
   , nickSplit
   , computeMaxMessageLength
   , capCmdText
+
+  , Source(..)
   ) where
 
 import           Control.Monad
@@ -47,18 +50,19 @@ import           View
 -- | High-level IRC message representation
 data IrcMsg
   = UnknownMsg !RawIrcMsg -- ^ pass-through for unhandled messages
-  | Reply !ReplyCode [Text] -- ^ code arguments
-  | Nick !UserInfo !Identifier -- ^ old new
-  | Join !UserInfo !Identifier !Text !Text -- ^ user channel account account gecos
-  | Part !UserInfo !Identifier (Maybe Text) -- ^ user channel reason
-  | Quit !UserInfo (Maybe Text) -- ^ user reason
-  | Kick !UserInfo !Identifier !Identifier !Text -- ^ kicker channel kickee comment
-  | Topic !UserInfo !Identifier !Text -- ^ user channel topic
-  | Privmsg !UserInfo !Identifier !Text -- ^ source target txt
-  | Ctcp !UserInfo !Identifier !Text !Text -- ^ source target command txt
-  | CtcpNotice !UserInfo !Identifier !Text !Text -- ^ source target command txt
-  | Notice !UserInfo !Identifier !Text -- ^ source target txt
-  | Mode !UserInfo !Identifier [Text] -- ^ source target txt
+  | Reply !Text !ReplyCode [Text] -- ^ server code arguments
+  | Nick !Source !Identifier -- ^ old new
+  | Join !Source !Identifier !Text !Text -- ^ user channel account account gecos
+  | Part !Source !Identifier (Maybe Text) -- ^ user channel reason
+  | Quit !Source (Maybe Text) -- ^ user reason
+  | Kick !Source !Identifier !Identifier !Text -- ^ kicker channel kickee comment
+  | Kill !Source !Identifier !Text -- ^ killer killee reason
+  | Topic !Source !Identifier !Text -- ^ user channel topic
+  | Privmsg !Source !Identifier !Text -- ^ source target txt
+  | Ctcp !Source !Identifier !Text !Text -- ^ source target command txt
+  | CtcpNotice !Source !Identifier !Text !Text -- ^ source target command txt
+  | Notice !Source !Identifier !Text -- ^ source target txt
+  | Mode !Source !Identifier [Text] -- ^ source target txt
   | Authenticate !Text -- ^ parameters
   | Cap !CapCmd -- ^ cap command and parameters
   | Ping [Text] -- ^ parameters
@@ -66,9 +70,13 @@ data IrcMsg
   | Error !Text -- ^ message
   | BatchStart !Text !Text [Text] -- ^ reference-id type parameters
   | BatchEnd !Text -- ^ reference-id
-  | Account !UserInfo !Text -- ^ user account name changed (account-notify extension)
-  | Chghost !UserInfo !Text !Text -- ^ Target, new username and new hostname
-  | Wallops  !UserInfo !Text -- ^ Braodcast message: Source, message
+  | Account !Source !Text -- ^ user account name changed (account-notify extension)
+  | Chghost !Source !Text !Text -- ^ Target, new username and new hostname
+  | Wallops  !Source !Text -- ^ Braodcast message: Source, message
+  | Invite !Source !Identifier !Identifier -- ^ sender target channel
+  deriving Show
+
+data Source = Source { srcUser :: {-# UNPACK #-}!UserInfo, srcAcct :: !Text }
   deriving Show
 
 data CapMore = CapMore | CapDone
@@ -97,13 +105,24 @@ cookCapCmd cmd args =
     ("DEL" , [     caps]) -> Just (CapDel (Text.words caps))
     _                     -> Nothing
 
+msgSource :: RawIrcMsg -> Maybe Source
+msgSource msg =
+  case view msgPrefix msg of
+    Nothing -> Nothing
+    Just p ->
+      case [a | TagEntry "account" a <- view msgTags msg ] of
+        []  -> Just (Source p "")
+        a:_ -> Just (Source p a)
+
+
 -- | Interpret a low-level 'RawIrcMsg' as a high-level 'IrcMsg'.
 -- Messages that can't be understood are wrapped in 'UnknownMsg'.
 cookIrcMsg :: RawIrcMsg -> IrcMsg
 cookIrcMsg msg =
   case view msgCommand msg of
-    cmd | Right (n,"") <- decimal cmd ->
-        Reply (ReplyCode n) (view msgParams msg)
+    cmd | Just user <- view msgPrefix msg
+        , Right (n,"") <- decimal cmd ->
+        Reply (idText (userNick user)) (ReplyCode n) (view msgParams msg)
     "CAP" | _target:cmdTxt:rest <- view msgParams msg
           , Just cmd <- cookCapCmd cmdTxt rest -> Cap cmd
 
@@ -113,50 +132,54 @@ cookIrcMsg msg =
     "PING" -> Ping (view msgParams msg)
     "PONG" -> Pong (view msgParams msg)
 
-    "PRIVMSG" | Just user <- view msgPrefix msg
+    "PRIVMSG" | Just source <- msgSource msg
            , [chan,txt]   <- view msgParams msg ->
 
            case parseCtcp txt of
-             Just (cmd,args) -> Ctcp user (mkId chan) (Text.toUpper cmd) args
-             Nothing         -> Privmsg user (mkId chan) txt
+             Just (cmd,args) -> Ctcp source (mkId chan) (Text.toUpper cmd) args
+             Nothing         -> Privmsg source (mkId chan) txt
 
-    "NOTICE" | Just user <- view msgPrefix msg
+    "NOTICE" | Just source <- msgSource msg
            , [chan,txt]    <- view msgParams msg ->
 
            case parseCtcp txt of
-             Just (cmd,args) -> CtcpNotice user (mkId chan) (Text.toUpper cmd) args
-             Nothing         -> Notice user (mkId chan) txt
+             Just (cmd,args) -> CtcpNotice source (mkId chan) (Text.toUpper cmd) args
+             Nothing         -> Notice source (mkId chan) txt
 
-    "JOIN" | Just user <- view msgPrefix msg
+    "JOIN" | Just source <- msgSource msg
            , chan:rest <- view msgParams msg
            , let (a, r) = case rest of
                             [acct, real] -> (acct, real)
                             _            -> ("", "") ->
-           Join user (mkId chan) a r
+           Join source (mkId chan) a r
 
-    "QUIT" | Just user <- view msgPrefix msg
+    "QUIT" | Just source <- msgSource msg
            , reasons   <- view msgParams msg ->
-           Quit user (listToMaybe reasons)
+           Quit source (listToMaybe reasons)
 
-    "PART" | Just user    <- view msgPrefix msg
+    "PART" | Just source <- msgSource msg
            , chan:reasons <- view msgParams msg ->
-           Part user (mkId chan) (listToMaybe reasons)
+           Part source (mkId chan) (listToMaybe reasons)
 
-    "NICK"  | Just user <- view msgPrefix msg
+    "NICK"  | Just source <- msgSource msg
             , newNick:_ <- view msgParams msg ->
-           Nick user (mkId newNick)
+           Nick source (mkId newNick)
 
-    "KICK"  | Just user <- view msgPrefix msg
+    "KICK"  | Just source <- msgSource msg
             , [chan,nick,reason] <- view msgParams msg ->
-           Kick user (mkId chan) (mkId nick) reason
+           Kick source (mkId chan) (mkId nick) reason
 
-    "TOPIC" | Just user <- view msgPrefix msg
+    "KILL"  | Just source <- msgSource msg
+            , [nick,reason] <- view msgParams msg ->
+           Kill source (mkId nick) reason
+
+    "TOPIC" | Just source <- msgSource msg
             , [chan,topic] <- view msgParams msg ->
-            Topic user (mkId chan) topic
+            Topic source (mkId chan) topic
 
-    "MODE"  | Just user <- view msgPrefix msg
+    "MODE"  | Just source <- msgSource msg
             , target:modes <- view msgParams msg ->
-            Mode user (mkId target) modes
+            Mode source (mkId target) modes
 
     "ERROR" | [reason] <- view msgParams msg ->
             Error reason
@@ -169,17 +192,21 @@ cookIrcMsg msg =
             , Just ('-',refid') <- Text.uncons refid ->
             BatchEnd refid'
 
-    "ACCOUNT" | Just user <- view msgPrefix msg
+    "ACCOUNT" | Just source <- msgSource msg
               , [acct] <- view msgParams msg ->
-      Account user (if acct == "*" then "" else acct)
+      Account source (if acct == "*" then "" else acct)
 
-    "CHGHOST" | Just user <- view msgPrefix msg
+    "CHGHOST" | Just source <- msgSource msg
               , [newuser, newhost] <- view msgParams msg ->
-      Chghost user newuser newhost
+      Chghost source newuser newhost
 
-    "WALLOPS" | Just user <- view msgPrefix msg
+    "WALLOPS" | Just source <- msgSource msg
               , [txt] <- view msgParams msg ->
-      Wallops user txt
+      Wallops source txt
+
+    "INVITE" | Just source <- msgSource msg
+             , [target, channel] <- view msgParams msg ->
+      Invite source (mkId target) (mkId channel)
 
     _      -> UnknownMsg msg
 
@@ -199,7 +226,6 @@ data MessageTarget
   = TargetUser   !Identifier -- ^ Metadata update for a user
   | TargetWindow !Identifier -- ^ Directed message to channel or from user
   | TargetNetwork            -- ^ Network-level message
-  | TargetHidden             -- ^ Completely hidden message
   deriving (Show)
 
 -- | Target information for the window that could be appropriate to
@@ -208,29 +234,31 @@ msgTarget :: Identifier -> IrcMsg -> MessageTarget
 msgTarget me msg =
   case msg of
     UnknownMsg{}             -> TargetNetwork
-    Nick user _              -> TargetUser (userNick user)
+    Nick user _              -> TargetUser (userNick (srcUser user))
     Mode _ tgt _ | tgt == me -> TargetNetwork
                  | otherwise -> TargetWindow tgt
     Join _ chan _ _          -> TargetWindow chan
     Part _ chan _            -> TargetWindow chan
-    Quit user _              -> TargetUser (userNick user)
+    Quit user _              -> TargetUser (userNick (srcUser user))
     Kick _ chan _ _          -> TargetWindow chan
+    Kill _ _ _               -> TargetNetwork
     Topic _ chan _           -> TargetWindow chan
-    Privmsg src tgt _        -> directed src tgt
-    Ctcp src tgt _ _         -> directed src tgt
-    CtcpNotice src tgt _ _   -> directed src tgt
-    Notice  src tgt _        -> directed src tgt
-    Authenticate{}           -> TargetHidden
-    Ping{}                   -> TargetHidden
-    Pong{}                   -> TargetHidden
+    Invite{}                 -> TargetNetwork
+    Privmsg src tgt _        -> directed (srcUser src) tgt
+    Ctcp src tgt _ _         -> directed (srcUser src) tgt
+    CtcpNotice src tgt _ _   -> directed (srcUser src) tgt
+    Notice  src tgt _        -> directed (srcUser src) tgt
+    Authenticate{}           -> TargetNetwork
+    Ping{}                   -> TargetNetwork
+    Pong{}                   -> TargetNetwork
     Error{}                  -> TargetNetwork
     Cap{}                    -> TargetNetwork
-    Reply code args          -> replyTarget code args
-    BatchStart{}             -> TargetHidden
-    BatchEnd{}               -> TargetHidden
-    Account user _           -> TargetUser (userNick user)
-    Chghost user _ _         -> TargetUser (userNick user)
-    Wallops src _            -> TargetWindow (userNick src)
+    Reply _ code args        -> replyTarget code args
+    BatchStart{}             -> TargetNetwork
+    BatchEnd{}               -> TargetNetwork
+    Account user _           -> TargetUser (userNick (srcUser user))
+    Chghost user _ _         -> TargetUser (userNick (srcUser user))
+    Wallops _ _              -> TargetNetwork
   where
     directed src tgt
       | Text.null (userHost src) = TargetNetwork -- server message
@@ -242,7 +270,7 @@ msgTarget me msg =
     replyTarget _                _        = TargetNetwork
 
 -- | 'UserInfo' of the user responsible for a message.
-msgActor :: IrcMsg -> Maybe UserInfo
+msgActor :: IrcMsg -> Maybe Source
 msgActor msg =
   case msg of
     UnknownMsg{}  -> Nothing
@@ -252,8 +280,10 @@ msgActor msg =
     Part x _ _    -> Just x
     Quit x _      -> Just x
     Kick x _ _ _  -> Just x
+    Kill x _ _    -> Just x
     Topic x _ _   -> Just x
     Privmsg x _ _ -> Just x
+    Invite x _ _  -> Just x
     Ctcp x _ _ _  -> Just x
     CtcpNotice x _ _ _ -> Just x
     Notice x _ _  -> Just x
@@ -269,34 +299,40 @@ msgActor msg =
     Chghost x _ _ -> Just x
     Wallops x _   -> Just x
 
+renderSource :: Source -> Text
+renderSource (Source u "") = renderUserInfo u
+renderSource (Source u a) = renderUserInfo u <> "(" <> a <> ")"
+
 -- | Text representation of an IRC message to be used for matching with
 -- regular expressions.
 ircMsgText :: IrcMsg -> Text
 ircMsgText msg =
   case msg of
     UnknownMsg raw -> Text.unwords (view msgCommand raw : view msgParams raw)
-    Reply (ReplyCode n) xs -> Text.unwords (Text.pack (show n) : xs)
-    Nick x y       -> Text.unwords [renderUserInfo x, idText y]
-    Join x _ _ _   -> renderUserInfo x
-    Part x _ mb    -> Text.unwords (renderUserInfo x : maybeToList mb)
-    Quit x mb      -> Text.unwords (renderUserInfo x : maybeToList mb)
-    Kick x _ z r   -> Text.unwords [renderUserInfo x, idText z, r]
-    Topic x _ t    -> Text.unwords [renderUserInfo x, t]
-    Privmsg x _ t  -> Text.unwords [renderUserInfo x, t]
-    Ctcp x _ c t   -> Text.unwords [renderUserInfo x, c, t]
-    CtcpNotice x _ c t -> Text.unwords [renderUserInfo x, c, t]
-    Notice x _ t   -> Text.unwords [renderUserInfo x, t]
-    Mode x _ xs    -> Text.unwords (renderUserInfo x:"set mode":xs)
+    Reply srv (ReplyCode n) xs -> Text.unwords (srv : Text.pack (show n) : xs)
+    Nick x y       -> Text.unwords [renderSource x, idText y]
+    Join x _ _ _   -> renderSource x
+    Part x _ mb    -> Text.unwords (renderSource x : maybeToList mb)
+    Quit x mb      -> Text.unwords (renderSource x : maybeToList mb)
+    Kick x _ z r   -> Text.unwords [renderSource x, idText z, r]
+    Kill x z r     -> Text.unwords [renderSource x, idText z, r]
+    Topic x _ t    -> Text.unwords [renderSource x, t]
+    Privmsg x _ t  -> Text.unwords [renderSource x, t]
+    Ctcp x _ c t   -> Text.unwords [renderSource x, c, t]
+    CtcpNotice x _ c t -> Text.unwords [renderSource x, c, t]
+    Notice x _ t   -> Text.unwords [renderSource x, t]
+    Mode x _ xs    -> Text.unwords (renderSource x:"set mode":xs)
     Ping xs        -> Text.unwords xs
     Pong xs        -> Text.unwords xs
     Cap cmd        -> capCmdText cmd
     Error t        -> t
-    Account x a    -> Text.unwords [renderUserInfo x, a]
+    Account x a    -> Text.unwords [renderSource x, a]
     Authenticate{} -> ""
     BatchStart{}   -> ""
     BatchEnd{}     -> ""
-    Chghost x a b  -> Text.unwords [renderUserInfo x, a, b]
-    Wallops x t    -> Text.unwords [renderUserInfo x, t]
+    Invite _ _ _   -> ""
+    Chghost x a b  -> Text.unwords [renderSource x, a, b]
+    Wallops x t    -> Text.unwords [renderSource x, t]
 
 capCmdText :: CapCmd -> Text
 capCmdText cmd =
